var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhInlineEditable', function($timeout, templateManager, maskEngine, ZhUtil) {
	return {
		link: function($scope, elem, attrs) {
			var column = $scope.$eval(attrs.zhInlineEditable);
			if (column && column.inlineEditable) {
				$timeout(function() {
					var elementContainer = elem.parent("div").parent("div");
					elementContainer.addClass('inline-editable');

					elementContainer.on('click.editable', function(e) {
						$scope.widget.inlineEditing = true;
						$scope.widget.lastInlineEditingElement = elementContainer;

						var input = elem;

						if (!input[0]._initialized) {
							input.on('blur', function(e) {
								var currentRow = $scope.widget.currentRow;

								var rowValue;
								var inputValue;
								if (column.mask) {
									rowValue = currentRow[column.name];
									inputValue = currentRow[column.name + "_inline"];
								} else {
									//Use the old behaviour for users that rely on the zhInlineEditable: "number" behaviour
									rowValue = getParsedValue(currentRow[column.name], column);
									inputValue = getParsedValue(input.val(), column);
								}

								var stringRowValue = Util.isUndefinedOrNull(rowValue) ? "": rowValue.toString();
								var stringInputValue = Util.isUndefinedOrNull(inputValue) ? "": inputValue.toString();

								if (stringRowValue != stringInputValue) {
									// put the widget in editing mode only if there is something changed
									$scope.widget.inlineEdit();
									templateManager.updateTemplate();

									if (Util.isUndefined(currentRow['changed_' + column.name])) {
										currentRow['changed_' + column.name] = currentRow[column.name];
									}

									if (!column.mask && isNumber(column)) {
										currentRow[column.name + "_inline"] = (isNaN(inputValue) || !inputValue) ? undefined : parseFloat(inputValue);
									}

									currentRow.changed = true;
									currentRow[column.name] = currentRow[column.name + "_inline"];
									$scope.row[column.name] = currentRow[column.name + "_inline"];

									$timeout(function() {
										$(window).trigger('resize.grid', $scope.widget.id);
									}, 0, false);
								} else if (!$scope.widget.inInlineEditMode()) {
									$scope.widget.inlineEditing = false;
								}

								var evtData = {
									data: {
										field: column
									}
								};
								if (column._isLastInlineEditable && $scope.widget.beforeInlineEditMoveRow) {
									$scope.widget.beforeInlineEditMoveRow($scope.widget.currentRow);
								}
								if (column.onInlineEdit) {
									column.onInlineEdit(evtData);
								}
								input.css('display', 'none');
							});

							input[0].__scope = $scope;

							input[0]._initialized = true;
						}

						input[0].cancelEditing = function() {
							$scope.widget.currentRow[column.name + "_inline"] = $scope.widget.currentRow[column.name];
							input.val($scope.widget.currentRow[column.name]);
							input.blur();
						};

						$timeout(function() {
							if ($scope.widget.currentRow != $scope.row) {
								$scope.widget.setCurrentRow($scope.row);
							}
	
							if ($scope.widget.currentRow[column.name + "_inline"]) {
								input.val($scope.widget.currentRow[column.name + "_inline"]);
							} else {
								$scope.widget.currentRow[column.name + "_inline"] = $scope.widget.currentRow[column.name];
								$scope.row[column.name + "_inline"] = $scope.widget.currentRow[column.name];
								var val = $scope.widget.currentRow[column.name];
								input.val(val);
							}
	
							input.css('display', 'block');
	
							horizontalScrollToColumn(input);
							Util.selectElement(input);
	
							templateManager.updateTemplate();
						}, 0, false);

						e.stopPropagation();
					});

					$(window).off('resize.grid.after');
					$(window).on('resize.grid.after', function(event) {
						if ($scope.widget.inlineEditing) {
							horizontalScrollToColumn($scope.widget.lastInlineEditingElement);
						}
					});
				});
			}

			function getClosestParentWithBodyScroll(inputElement) {
				return inputElement.closest('.container').parent().parent().find('.body-scroll-control');
			}

			function getClosestParentWithVerticalScroll(inputElement) {
				return inputElement.closest('.container').parent().parent().find('.grid-vertical-scroll-control');
			}

			function horizontalScrollToColumn(inputElement){
				var elementPosition = inputElement.offset().left - inputElement.closest('.zh-grid-container').offset().left;
				var closestGrid = inputElement.closest('.container');
				var gridLeftPadding = parseInt(closestGrid.css("paddingLeft").replace("px", ""));
				if((elementPosition + inputElement.width()) > closestGrid.width() || (elementPosition - gridLeftPadding) <= 0){
					var closestVerticalScroller = getClosestParentWithVerticalScroll(inputElement);
					var closestHorizontalScroller = getClosestParentWithBodyScroll(inputElement);
					var notVisiblePart;
					var elementLeftPadding = parseInt(inputElement.css("paddingLeft").replace("px", ""));
					if((elementPosition - gridLeftPadding) <= 0){
						notVisiblePart = - (elementLeftPadding + closestVerticalScroller.width());
					} else {
						notVisiblePart = inputElement.width() + closestVerticalScroller.width() - closestGrid.width() + elementLeftPadding;
					}
					var notVisiblePosition = elementPosition + notVisiblePart;
					var pxToScroll = closestHorizontalScroller.scrollLeft() + notVisiblePosition;
					closestHorizontalScroller.scrollLeft(pxToScroll);
				}
			}
			function isNumber(column) {
				return column.inlineEditable === 'number';
			}

			function getParsedValue(value, column) {
				return (value && isNumber(column)) ? value.toString().replace(",", ".") : value;
			}

			$scope.$on('$destroy', function() {
				delete elem[0].__scope;
				var elementContainer = elem.parent("div").parent("div");
				elementContainer.off('click.editable');
			});
		}
	};
});

var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhInlineSelectNative', function($timeout, templateManager, maskEngine, ZhUtil) {
	return {
		link: function($scope, elem, attrs) {
			var column = $scope.$eval(attrs.zhInlineSelectNative);
			if (column && column.inlineEditable === "select-native") {
				$timeout(function() {
					var elementContainer = elem.parents("div.td");
					elementContainer.addClass('inline-editable');

					var clickEditable = function(e) {
						if ($(e.target).is('li.option')) return;
						
						$scope.widget.inlineEditing = true;
						$scope.widget.lastInlineEditingElement = elementContainer;

						var select = elem;
						var newSelect = elem.siblings('.new-select');

						if (!select[0]._initialized) {
							newSelect.on('blur', function(e) {
								var currentRow = $scope.widget.currentRow;

								var rowValue;
								var selectValue;
								rowValue = currentRow[column.name];
								selectValue = currentRow[column.name + "_inline"];

								var stringRowValue = Util.isUndefinedOrNull(rowValue) ? "": rowValue.toString();
								var stringSelectValue = Util.isUndefinedOrNull(selectValue) ? "": selectValue.toString();

								if (stringRowValue != stringSelectValue) {
									// put the widget in editing mode only if there is something changed
									$scope.widget.inlineEdit();
									templateManager.updateTemplate();

									if (Util.isUndefined(currentRow['changed_' + column.name])) {
										currentRow['changed_' + column.name] = currentRow[column.name];
									}

									currentRow.changed = true;
									currentRow[column.name] = currentRow[column.name + "_inline"];
									$scope.row[column.name] = currentRow[column.name + "_inline"];

									$timeout(function() {
										$(window).trigger('resize.grid', $scope.widget.id);
									}, 0, false);
								} else if (!$scope.widget.inInlineEditMode()) {
									$scope.widget.inlineEditing = false;
								}

								var evtData = {
									data: {
										field: column
									}
								};
								if (column._isLastInlineEditable && $scope.widget.beforeInlineEditMoveRow) {
									$scope.widget.beforeInlineEditMoveRow($scope.widget.currentRow);
								}
								if (column.onInlineEdit) {
									column.onInlineEdit(evtData);
								}
								newSelect.css('display', 'none');
							});

							select[0].__scope = $scope;

							select[0]._initialized = true;
						}

						newSelect[0].cancelEditing = function() {
							$scope.widget.currentRow[column.name + "_inline"] = $scope.widget.currentRow[column.name];
							select.val($scope.widget.currentRow[column.name]);
							newSelect.blur();
						};

						$timeout(function() {
							if ($scope.widget.currentRow != $scope.row) {
								$scope.widget.setCurrentRow($scope.row);
							}
	
							if ($scope.widget.currentRow[column.name + "_inline"]) {
								select.val($scope.widget.currentRow[column.name + "_inline"]);
							} else {
								$scope.widget.currentRow[column.name + "_inline"] = $scope.widget.currentRow[column.name];
								$scope.row[column.name + "_inline"] = $scope.widget.currentRow[column.name];
								var val = $scope.widget.currentRow[column.name];
								column.value("");
								column.value(val);
							}
	
							if (newSelect.css('display') !== 'block') {
								newSelect.css('display', 'block');
		
								horizontalScrollToColumn(newSelect);
								Util.selectElement(newSelect);
		
								templateManager.updateTemplate();
		
								elementContainer.off('click.editable');
								newSelect.click();
								elementContainer.on('click.editable', clickEditable);
							}
						}, 0, false);
					};
					
					elementContainer.on('click.editable', clickEditable);

					$(window).off('resize.grid.after');
					$(window).on('resize.grid.after', function(event) {
						if ($scope.widget.inlineEditing) {
							horizontalScrollToColumn($scope.widget.lastInlineEditingElement);
						}
					});
				});
			}

			function getClosestParentWithBodyScroll(inputElement) {
				return inputElement.closest('.container').parent().parent().find('.body-scroll-control');
			}

			function getClosestParentWithVerticalScroll(inputElement) {
				return inputElement.closest('.container').parent().parent().find('.grid-vertical-scroll-control');
			}

			function horizontalScrollToColumn(inputElement){
				var elementPosition = inputElement.offset().left - inputElement.closest('.zh-grid-container').offset().left;
				var closestGrid = inputElement.closest('.container');
				var gridLeftPadding = parseInt(closestGrid.css("paddingLeft").replace("px", ""));
				if((elementPosition + inputElement.width()) > closestGrid.width() || (elementPosition - gridLeftPadding) <= 0){
					var closestVerticalScroller = getClosestParentWithVerticalScroll(inputElement);
					var closestHorizontalScroller = getClosestParentWithBodyScroll(inputElement);
					var notVisiblePart;
					var elementLeftPadding = parseInt(inputElement.css("paddingLeft").replace("px", ""));
					if((elementPosition - gridLeftPadding) <= 0){
						notVisiblePart = - (elementLeftPadding + closestVerticalScroller.width());
					} else {
						notVisiblePart = inputElement.width() + closestVerticalScroller.width() - closestGrid.width() + elementLeftPadding;
					}
					var notVisiblePosition = elementPosition + notVisiblePart;
					var pxToScroll = closestHorizontalScroller.scrollLeft() + notVisiblePosition;
					closestHorizontalScroller.scrollLeft(pxToScroll);
				}
			}

			$scope.$on('$destroy', function() {
				delete elem[0].__scope;
				var elementContainer = elem.parent("div").parent("div");
				elementContainer.off('click.editable');
			});
		}
	};
});
